/* ============================= C MeatAxe ==================================
   File:        $Id: chbasis.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Change basis.
   --------------------------------------------------------------------------
   (C) Copyright 1997 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"

MTX_DEFINE_FILE_INFO


/*{{{ --------------------------------------------------------------------------
!section obj.matrep
!function RepChangeBasis "Change basis"
!arg rep
    Matrix representation.
!arg trans   
    Basis transformation matrix.
!return
    $0$ on success, $-1$ on error.
!description
    This function performs a change of basis on a matrix representation.
    |trans| is the transformation matrix $T$. The rows of |trans| must 
    contain the new basis vectors, expressed in the old basis. Then, the 
    transformed generators are given by
    \[
	g_i' = T g_i T^{-1}
    \]
!seealso 
-------------------------------------------------------------------------- }}}*/

int RepChangeBasis(Rep_t *rep, const Matrix_t *trans)
{
    Matrix_t *bi;
    int i;

    /* Check arguments
       --------------- */
    if (!RepIsValid(rep))
    {
	MTX_ERROR1("rep: %E",MTX_ERR_BADARG);
	return -1;
    }
    if (!MatIsValid(trans))
    {
	MTX_ERROR1("trans: %E",MTX_ERR_BADARG);
	return -1;
    }
    if (rep->NGen <= 0)
	return 0;
    if (   !RepCanMap(rep,trans->Field,trans->Noc)
	|| trans->Nor != trans->Noc)
    {
	MTX_ERROR1("%E",MTX_ERR_INCOMPAT);
	return -1;
    }


    /* Basis transformation
       -------------------- */
    if ((bi = MatInverse(trans)) == NULL) 
    {
	MTX_ERROR("Basis transformation is singular");
	return -1;
    }
    for (i = 0; i < rep->NGen; ++i)
    {
	Matrix_t *tmp = MatDup(trans);
	MtxMul(tmp,rep->Gen[i]);
	MatMul(tmp,bi);
        MtxFree(rep->Gen[i]);
	rep->Gen[i] = (MtxObject_t *) tmp;
    }
    MatFree(bi);
    return 0;
}



int ChangeBasisOLD(const Matrix_t *M, int ngen, MtxObject_t *gen[],
	MtxObject_t *newgen[])

{
    Matrix_t *bi, *tmp;
    int i;

    MTX_VERIFY(ngen >= 0);
    if (!MatIsValid(M))
	return -1;
    if ((bi = MatInverse(M)) == NULL) 
    {
	MTX_ERROR("Matrix is singular");
	return -1;
    }
    for (i = 0; i < ngen; ++i)
    {
	tmp = MatDup(M);
	MtxMul(tmp,gen[i]);
	MatMul(tmp,bi);
	if (newgen == gen)
	    MtxFree(newgen[i]);
	newgen[i] = (MtxObject_t *) tmp;
    }
    MatFree(bi);
    return 0;
}


