/* ============================= C MeatAxe ==================================
   File:        $Id: isisom.c,v 1.2 2000/03/18 13:58:41 mringe Exp $
   Comment:     Isomorphism test.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"


MTX_DEFINE_FILE_INFO



static int CheckArgs(const Rep_t *rep1, const CfInfo *info1, const Rep_t *rep2, int use_pw)
{
    int j;

    /* Check arguments
       --------------- */
    if (!RepIsValid(rep1))
	return -1;
    if (!RepIsValid(rep2))
	return -1;
    MTX_VERIFY(rep1->NGen == rep2->NGen);
    MTX_VERIFY(rep1->NGen > 0);
    MTX_VERIFY(rep1->Gen[0]->Type = MTX_MATRIX);
    MTX_VERIFY(rep2->Gen[0]->Type = MTX_MATRIX);

    for (j = 0; j < rep1->NGen; ++j)
    {
	const Matrix_t *mat1 = (const Matrix_t *)rep1->Gen[j];
	const Matrix_t *mat2 = (const Matrix_t *)rep2->Gen[j];

        if (!MatIsValid(mat1) || !MatIsValid(mat2))
	    return -1;
	MTX_VERIFY(mat1->Nor == mat1->Noc);
	MTX_VERIFY(mat2->Nor == mat2->Noc);
	MTX_VERIFY(mat1->Field == mat2->Field);
        MTX_VERIFY(info1->dim == mat1->Nor);
    }

    if (use_pw && info1->peakword <= 0)
    {
	MTX_ERROR("No peak word available");
	return -1;
    }
    if (!use_pw && info1->idword <= 0)
    {
	MTX_ERROR("No id word available");
	return -1;
    }
    return 0;
}



/*{{{ --------------------------------------------------------------------------
!section algo.etc
!function IsIsomorphic "Compare representations"
!arg rep1
    The first representation.
!arg info1
    Pointer to an |CfInfo| structure for the first representation.
!arg rep2
    The second representation.
!arg trans
    Buffer for basis transformation matrix, or |NULL|.
!arg use_pw
    If different from zero, use peak word instead of the identifying word 
    (see below).
!return
    1 if the representations are isomorphic, 0 if they are not isomorphic, and
    $-1$ on error.
!description
    This function decides if two irreducible representations are isomorphic.
    |rep1| and |rep2| must be two matrix representations over the same field,
    and with the same number of generators. Furthermore,
    to compare the representations, the function needs an identifying word
    for the first representation, i.e., the fields |info1->idword|, 
    |info1->idpol| and |info1->spl| must be set, and the generators in 
    |rep1| must be in standard basis with respect to the identifying word.
    If |use_pw| is nonzero, the peak word is used instead of the idword.
    In this case, |rep1| must of course be in standard basis with respect 
    to the peak word.

    If the representations are isomorphic, and |trans| is not |NULL|, the
    basis transformation which makes the second representation identical
    to the first is stored into |*trans|. To be more precise, if $g_i$ is 
    the representation of the i-th generator in representation |rep1|,
    $h_i$ in representation |rep2|, and $T$ the 
    matrix returned in |trans|, then $Th_i T^{-1}=g_i$.
!seealso 
-------------------------------------------------------------------------- }}}*/

int IsIsomorphic(const Rep_t *rep1, const CfInfo *info1,
    const Rep_t *rep2, Matrix_t  **trans, int use_pw)

{
    int j;
    WgData_t *wg;
    Matrix_t *word, *m, *seed, *b, *bi;
    int result;

    if (CheckArgs(rep1,info1,rep2,use_pw) != 0)
	return -1;
 
    /* Make the idword on representation 2
       ----------------------------------- */
    wg = WgAlloc(rep2);
    word = WgMakeWord(wg,use_pw ? info1->peakword : info1->idword);
    m = MatInsert(word,use_pw ? info1->peakpol : info1->idpol);
    MatFree(word);
    WgFree(wg);
    seed = MatNullSpace__(m);
    if (seed->Nor != info1->spl)
    {	
	MatFree(seed);
	return 0;
    }

    /* Make the standard basis
       ----------------------- */
    b = SpinUp(seed,rep2,SF_FIRST|SF_CYCLIC|SF_STD,NULL,NULL);
    MatFree(seed);
    if (b->Nor != b->Noc)
    {
	MatFree(b);
	return 0;
    }
    bi = MatInverse(b);

    /* Compare generators
       ------------------ */
    for (j = 0, result = 0; result == 0 && j < rep2->NGen; ++j)
    {
	Matrix_t *g = MatDup(b);
	/* OLD: MatMul(g,rep2->Gen[j]); */
	MtxMul(g,rep2->Gen[j]);
	MatMul(g,bi);
	if (MatCompare(g,(const Matrix_t *)rep1->Gen[j]) != 0)
	    result = 1;
	MatFree(g);
    }

    /* Clean up 
       -------- */
    if (trans != NULL && result == 0)
	*trans = b;
    else
	MatFree(b);
    MatFree(bi);

    return (result == 0);
}
