/* ============================= C MeatAxe ==================================
   File:        $Id: mat2sparse.c,v 1.5 2003/05/21 23:02:44 hoffmant Exp $
   Comment:     Convert a mtx matrix into a row sparse matrix.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   Modified by Tom Hoffman, Mathematics Department, University of
   Arizona, USA <hoffmant@math.arizona.edu>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include <string.h>
#include <stdlib.h>
#include "meataxe.h"




/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

MTX_DEFINE_FILE_INFO

static const char *iname, *oname;
static MtxFile_t *InputFile = NULL;
static MtxFile_t *out = NULL;	    /* Output file */
static int nonzero;                 /* number of nonzero entries in output */
typedef unsigned char BYTE;
static int MPB = 0;             /* No. of marks per byte */


static MtxApplicationInfo_t AppInfo = { 
"mat2sparse", "Convert MeatAxe matrix into a row sparse format", 
"SYNTAX\n"
"    mat2sparse " MTX_COMMON_OPTIONS_SYNTAX " <col> <Input> <Output>\n"
"\n"
"OPTIONS\n"
MTX_COMMON_OPTIONS_DESCRIPTION
"\n"
"ARGUMENTS\n"
"    <col> ................... bound for number of nonzero entries\n"
"    <Input> ................. Input file name\n"
"    <Output> ................ Output file name\n"
};

static MtxApplication_t *App = NULL;




static int Convert()
{
    PTR m1, m2;
    register BYTE *r;
    long hdr[2];
    long *row;
    FEL f1;
    long loop1, j1, j2, i;

    if ((out = MfCreateAndWriteHeader(oname,-7,InputFile->Nor,InputFile->Noc)) == NULL)
        return -1;
    hdr[0]=InputFile->Field;
    hdr[1]=nonzero;
    MfWriteLong(out,hdr,2);

    i=InputFile->Field;
    while (i < 257)
      {
        i=i*InputFile->Field;
	MPB+=1;
      }

    /* Output sparse matrix */
    FfSetField(InputFile->Field);
    FfSetNoc(nonzero);
    m2 = FfAlloc(1);

    /* Input matrix */
    FfSetNoc(InputFile->Noc);
    m1 = FfAlloc(1);

    MESSAGE(0,("sparse-matrix field=%d rows=%d cols=%d nonzero=%d\n",
	   InputFile->Field,InputFile->Nor,InputFile->Noc,nonzero));
    for (loop1 = 1; loop1 <= InputFile->Nor; ++loop1)
    {   
        row = NALLOC(long,nonzero);
	/* initialize row */
	for(i=0; i < nonzero; i++)
	    row[i]=1;
        FfSetNoc(InputFile->Noc);
	if (MfReadRows(InputFile,m1,1) != 1)
        {
            MTX_ERROR1("Cannot read %s",InputFile);
            return -1;
        }
	j2=0;
	r = (BYTE *) m1;
        for (j1 = 0; j1 < InputFile->Noc; ++r)
	  {
	    if (*r==0)
	      {
		j1+=MPB;
	      }
	    else
	      {
		for (i = 0; i < MPB && j1 < InputFile->Noc; i++)
		  {
		    FfSetNoc(InputFile->Noc);
		    f1 = FfExtract(m1,j1);
		    if (f1 == FF_ZERO)
		      {
			/* do nothing */
		      }
		    else
		      {
			if (j2 >= nonzero)
			  {
			    printf("Matrix is not %d sparse!\n",nonzero);
			    return -1;
			  }
			FfSetNoc(nonzero);
			FfInsert(m2,j2,f1);
			row[j2]=j1+1;
			++j2;
		      }
		    j1+=1;
		  }
	      }
	  }
	FfSetNoc(nonzero);
	for (i = j2; i < nonzero; i++)
	{
	  FfInsert(m2,i,FF_ZERO);
	  row[i]=0;
	}
        if (MfWriteRows(out,m2,1) != 1)
        {
            MTX_ERROR1("Cannot write %s",oname);
            return -1;
        }
	MfWriteLong(out,row,nonzero);

        FfMulRow(m2,FF_ZERO);
	SysFree(row);
    }
    return 0;
}







static int Init(int argc, const char **argv)

{
    /* Process command line options and arguments.
       ------------------------------------------- */
    if ((App = AppAlloc(&AppInfo,argc,argv)) == NULL)
	return -1;
    if (AppGetArguments(App,3,3) != 3)
	return -1;
    nonzero = atol(App->ArgV[0]);
    iname = App->ArgV[1];
    oname = App->ArgV[2];

    return 0;
}



static void Cleanup()

{
    if (App != NULL)
	AppFree(App);
    if (InputFile != NULL)
	MfClose(InputFile);
    if (out != NULL)
	MfClose(out);
}



/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, const char **argv)

{
    int rc = 0;

    if (Init(argc,argv) != 0)
	return -1;

    if ((InputFile = MfOpenAndReadHeader(iname)) == NULL)
	rc = -1;

    /* Convert
       ------- */
    Convert();

    Cleanup();

    return rc;
}

/*{{{ --------------------------------------------------------------------------
!section apps.futil
!program mat2sparse "Convert MeatAxe matrix into a row sparse format"
!synopsis 
    mat2sparse <q> <Input> <Output>
!arg q
    The bound on the number of nonzero entries per row in a sparse matrix.
!arg Input
    Input file name.
!arg Output
    Output file name.
!seealso 
    sparse2mat
!description
    This program converts MeatAxe matrices into row sparse format matrices.
-------------------------------------------------------------------------- }}}*/

