/* ============================= C MeatAxe ==================================
   File:        $Id: matmul.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Matrix multiplication.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"


/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO 




/*{{{ --------------------------------------------------------------------------
!section obj.mat
!function MatMul "Multiply matrices"
!arg dest
    Left factor.
!arg src
    Right factor.
!return
    The function returns |dest|, or |NULL| on error.
!description
    This function multiplies the matrix |dest| from the right by the matrix 
    |src|.
    The matrices must be compatible for multiplication, i.e. they must be over
    the same field, and the number of columns of |dest| must be equal to the 
    number f rows of |src|.
    The result of the multiplication is stored in |dest|, overwriting the
    original contents.
!seealso MatPower
-------------------------------------------------------------------------- }}}*/

Matrix_t *MatMul(Matrix_t *dest, const Matrix_t *src)
{
    PTR x, tmp, result;
    long i;

    /* Check the arguments
       ------------------- */
#ifdef DEBUG
    if (!MatIsValid(src) || !MatIsValid(dest))
    {
	MTX_ERROR1("%E",MTX_ERR_BADARG);
	return NULL;
    }
    if (src->Field != dest->Field || src->Nor != dest->Noc)
    {
	MTX_ERROR7("Can't multiply %dx%d/GF(%d) by %dx%d/GF(%d): %E",
	    dest->Nor,dest->Noc,dest->Field,src->Nor,src->Noc,src->Field,
	    MTX_ERR_INCOMPAT);
	return NULL;
    }
#endif

    /* Matrix multiplication
       --------------------- */
    FfSetField(src->Field);
    FfSetNoc(src->Noc);
    result = tmp = FfAlloc(dest->Nor);
    if (result == NULL)
    {
	MTX_ERROR("MatMul(): Cannot allocate product");
	return NULL;
    }
    x = dest->Data;
    for (i = 0; i < dest->Nor; ++i)
    {
	FfMapRow(x,src->Data,src->Nor,tmp);
	FfStepPtr(&tmp);
/*	x = FfGetPtr(x,1,dest->Noc);*/
	x = (PTR)((char*) x + dest->RowSize);
    }
    SysFree(dest->Data);
    dest->Data = result;
    dest->Noc = src->Noc;
    dest->RowSize = FfCurrentRowSize;

    Mat_DeletePivotTable(dest);

    return dest;
}


int MtxMul(Matrix_t *mat, const MtxObject_t *gen)
{
    PTR tmp, result;
    long i;
    int gnoc;

    /* Check the arguments
       ------------------- */
    if (!MatIsValid(mat))
	return -1;
    if (!MtxCanMap(gen,mat->Field,mat->Noc))
        return -1;

    /* Matrix multiplication
       --------------------- */
    gnoc = MtxGetNoc(gen);
    FfSetField(mat->Field);
    FfSetNoc(gnoc);
    result = tmp = FfAlloc(mat->Nor);
    if (result == NULL)
    {
	MTX_ERROR("Cannot allocate buffer");
	return -1;
    }
    for (i = 0; i < mat->Nor; ++i)
    {
	MtxMapRow(MatGetPtr(mat,i),gen,tmp);
        FfSetNoc(gnoc);
	FfStepPtr(&tmp);
    }

    /* Replace <mat>'s data with the result.
       ------------------------------------- */
    SysFree(mat->Data);
    mat->Data = result;
    mat->Noc = gnoc;
    mat->RowSize = FfCurrentRowSize;
    Mat_DeletePivotTable(mat);

    return 0;
}



