/* ============================= C MeatAxe ==================================
   File:        $Id: matread.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Read a matrix from a file.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <stdlib.h>

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.mat
!function MatRead "Read a matrix from a file"
!synopsis 
    Matrix_t *MatRead(FILE *f);
!arg f
    File to read from.
!return
    Pointer to the matrix, or |NULL| on error.
!description
    This function opens reads a matrix from a file.
!seealso MatLoad
-------------------------------------------------------------------------- }}}*/

Matrix_t *MatRead(MtxFile_t *f)
{
    Matrix_t *m;

    if (MfReadHeader(f) != 0) 
	return NULL;
    if (f->Type != MTX_MATRIX) 
    {
	MTX_ERROR1("%E",MTX_ERR_NOTMATRIX);
	return NULL;
    }
    m = MatAlloc(f->Field,f->Nor,f->Noc);
    if (m == NULL) 
	return NULL;
    if (MfReadRows(f,m->Data,m->Nor) != m->Nor)
    {
	MatFree(m);
	return NULL;
    }
    return m;
}



/*{{{ --------------------------------------------------------------------------
!function MatLoad "Read a matrix from a file"
!synopsis 
    Matrix_t *MatLoad(const char *fn);
!arg fn
    File name.
!return
    Pointer to the matrix, or |NULL| on error.
!description
    This function opens a file, reads a single matrix, and closes the file.

    To read more than one matrix from a file, use |MatRead()|.
!seealso MatRead
-------------------------------------------------------------------------- }}}*/

Matrix_t *MatLoad(const char *fn)
{
    MtxFile_t *f;
    Matrix_t *m;

    if ((f = MfOpen(fn)) == NULL)
	return NULL;
    m = MatRead(f);
    MfClose(f);
    return m;
}



/* Called from MfReadHeader() */

int MatEvalHeader(MtxFile_t *f, long hdr[3])
{
    f->Type = MTX_MATRIX;
    f->Field = (int) hdr[0];
    f->Nor = (int) hdr[1];
    f->Noc = (int) hdr[2];
    FfSetField(f->Field);
    FfSetNoc(f->Noc);
    f->DataSize = f->Noc * FfTrueRowSize(f->Noc);
    if (f->Nor < 0 || f->Noc < 0 || f->Field > 256)
    {
	MTX_ERROR1("%s: Invalid matrix header",f->FileName);
	return -1;
    }
    return 0;
}


    
