/* ============================= C MeatAxe ==================================
   File:        $Id: mfcore.c,v 1.2 2003/04/09 18:12:33 hoffmant Exp $
   Comment:     MeatAxe file objects: basic functions.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   Modified by Tom Hoffman, Mathematics Department, University of
   Arizona, USA <hoffmant@math.arizona.edu>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO

#define MF_MAGIC 0x229AE77B

   
   

/*{{{ --------------------------------------------------------------------------
!definesection obj.file
\index{MtxFile_t@{\tt MtxFile\protect\_t} data type}
The concept of a {\MeatAxe} data file with header and data part is encapsulated
in the |MtxFile_t| object.
-------------------------------------------------------------------------- }}}*/



/*{{{ --------------------------------------------------------------------------
!section obj.file
!function MfIsValid "Check a file object"
!arg file
    Pointer to the file.
!return
    1 if |file| points to a valid file objects, 0 otherwise.
!description
    This function checks if the argument |file| is a pointer to a valid
    {\MeatAxe} file object.
!seealso Matrix_t
-------------------------------------------------------------------------- }}}*/

int MfIsValid(const MtxFile_t *file)
{
    if (file == NULL)
    {
	MTX_ERROR("NULL file");
	return 0;
    }
    if (file->Magic != MF_MAGIC)
    {
	MTX_ERROR("Invalid file");
	return 0;
    }
    return 1;
}


static MtxFile_t *Mf_Alloc(const char *name)
{
    MtxFile_t *f;

    if ((f = ALLOC(MtxFile_t)) == NULL)
	return NULL;
    memset(f,0,sizeof(*f));
    strncpy(f->FileName,name,sizeof(f->FileName));
    return f;
}


static void Mf_Free(MtxFile_t *f)
{
    if (f->File != NULL)
	fclose(f->File);
    memset(f,0,sizeof(*f));
    SysFree(f);
}


MtxFile_t *MfOpen(const char *name)
{
    MtxFile_t *f;

    if ((f = Mf_Alloc(name)) == NULL)
	return NULL;
    if ((f->File = SysFopen(name,FM_READ)) == NULL)
    {
	Mf_Free(f);
	return NULL;
    }
    f->Magic = MF_MAGIC;
    return f;
}


MtxFile_t *MfOpenAndReadHeader(const char *name)
{
    MtxFile_t *f;

    if ((f = MfOpen(name)) == NULL)
	return NULL;

    /* Read the header */
    if (MfReadHeader(f) != 0)
    {
	Mf_Free(f);
	return NULL;
    }
    return f;
}



MtxFile_t *MfCreate(const char *name)
{
    MtxFile_t *f;
    if ((f = Mf_Alloc(name)) == NULL)
        return NULL;
    if ((f->File = SysFopen(name,FM_CREATE)) == NULL)
    {
	Mf_Free(f);
	return NULL;
    }
    f->Magic = MF_MAGIC;
    return f;
}






MtxFile_t *MfCreateAndWriteHeader(const char *name,
	int field, int nor, int noc)
{
    MtxFile_t *f;
    long header[3];

    if ((f = MfCreate(name)) == NULL)
        return NULL;

    /* Write the file header.
       ---------------------- */
    header[0] = f->Field = field;
    header[1] = f->Nor = nor;
    header[2]= f->Noc = noc;
    if (SysWriteLong(f->File,header,3) != 3)
    {
	MTX_ERROR1("%s: Error writing file header",name);
	Mf_Free(f);
	return NULL;
    }
    return f;
}




int MfClose(MtxFile_t *file)
{
    if (!MfIsValid(file))
	return -1;
    Mf_Free(file);
    return 0;
}



int MfReadHeader(MtxFile_t *f)
{
    long hdr[3];
    int error = 0;

    /* Reset all fields.
       ----------------- */
    f->Type = (MtxType_t) 0;
    f->Field = 0;
    f->Degree = 0;
    f->Nor = f->Noc = f->NonZero = 0;

    /* Read the fixed part of the header (three long integers).
       -------------------------------------------------------- */
    if (SysReadLong(f->File,hdr,3) != 3)
    {
	MTX_ERROR1("%s: Error reading file header",f->FileName);
	return -1;
    }

    /* Process the header, reading additional entries where needed.
       ------------------------------------------------------------ */
    if (hdr[0] >= 2)			/* Matrix */
	error = MatEvalHeader(f,hdr);
    else if (hdr[0] == -1)		/* Permutation */
	error = PermEvalHeader(f,hdr);
    else if (hdr[0] == -2)		/* Polynomial */
	error = PolEvalHeader(f,hdr);
    else if (hdr[0] == -7)		/* Sparse matrix */
	error = SMatEvalHeader(f,hdr);
    else
    {
#if 0
	MTX_ERROR2("%s: Unknown type %d in file header",name,(int)hdr[0]);
        return -1;
#else
	/* this is  temporary hack, until all types are recognized */
	f->Field = (int) hdr[0];
        f->Nor = (int) hdr[1];
        f->Noc = (int) hdr[2];
#endif

    }

    return error;
}



/* Like MfReadHeader(), but keeps file position unchanged */

int MfPeekHeader(MtxFile_t *file)
{
    int rc;
    fpos_t pos;

    fgetpos(file->File,&pos);	/* Save file pointer */
    rc = MfReadHeader(file);
    fsetpos(file->File,&pos);   /* Restore file pointer */
    return rc;
}


