/* ============================= C MeatAxe ==================================
   File:        $Id: kernel-0.c,v 1.2 2003/04/09 18:12:25 hoffmant Exp $
   Comment:     Map a vector by a generator (matrix, permutation, etc).
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"


/* ------------------------------------------------------------------
   Gobal data
   ------------------------------------------------------------------ */

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section ????
!function MtxMapRow "Multiply a vector by a generator"
!arg row
    The vector.
!arg gen
    The generator.
!arg result
    Pointer to a buffer where the result will be stored.
!description
    This function multiplies the vector |row| from the right by
    |gen| and stores the result in |result|.
    |gen| can be a matrix, sparse matrix, or permutation.

    The length of the input vector is determined implicitely by
    |gen|. For example, if |gen| is a matrix, the vector length
    ist equal to the number of rows of the matrix. Note that the
    function cannot verify that |row| has the correct size --- it
    is the callers responsibility to make sure that |row| and
    |gen| are compatible.

    Note: |result| and |row| must not overlap. Otherwise the result is
    undefined.
!seealso
-------------------------------------------------------------------------- }}}*/

void MtxMapRow(PTR row, const MtxObject_t *gen, PTR result)
{
    switch (gen->Type)
    {
	case MTX_MATRIX:
	    FfSetNoc(((const Matrix_t *)gen)->Noc);
	    FfMapRow(row,((const Matrix_t *)gen)->Data,
		((const Matrix_t *)gen)->Nor,result);
	    break;
	case MTX_PERMUTATION:
	    FfSetNoc(((const Perm_t *)gen)->Degree);
	    FfPermRow(row,((const Perm_t *)gen)->Data,result);
	    break;
	case MTX_SPARSE_MATRIX:
	    FfSMapRow(row,(const SparseMatrix_t*)gen,result);
	    break;
	default:
	    MTX_ERROR1("Unknown object type %d",gen->Type);
    }
}




void MtxExtractRow(PTR row, const MtxObject_t *gen, int i)
{
    switch (gen->Type)
    {
	case MTX_MATRIX:
	{
	    const Matrix_t *mat = (const Matrix_t *)gen;
	    FfSetNoc(mat->Noc);
	    FfCopyRow(row,MatGetPtr(mat,i));
	    break;
	}

	case MTX_SPARSE_MATRIX:
	{
	    int k;
	    const SparseMatrix_t *mat = (const SparseMatrix_t *) gen;
	    const long *pos = mat->Position + (mat->NonZero * i);
	    PTR srow = SMatGetPtr(mat,i);
	    FfSetNoc(mat->Noc);
	    FfMulRow(row,FF_ZERO);
	    for (k = 0; k < mat->NonZero; ++k)
	    {
		if (pos[k] != 0)
		    FfInsert(row,pos[k] - 1,FfExtract(srow,k));
	    }
	    break;
	}

	case MTX_PERMUTATION:
	{
	    const Perm_t *p = (const Perm_t *) gen;
	    FfSetNoc(p->Degree);
	    FfMulRow(row,FF_ZERO);
	    FfInsert(row,p->Data[i],FF_ONE);
	    break;
	}
	default:
	    MTX_ERROR1("Unknown object type %d",gen->Type);
    }
}









