/* ============================= C MeatAxe ==================================
   File:        $Id: mtxobj.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Polymorphic objects.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO
   

#define IS_MATRIX(x) (((Matrix_t *)(x))->Type == MTX_MATRIX)





MtxObject_t *MtxRead(MtxFile_t *f)
{
    long fl;

    if (SysReadLong(f->File,&fl,1) != 1)
    {
	MTX_ERROR("Error reading file header: %S");
	return NULL;
    }

    /* TODO: use relative positioning, MtxRead() should work
     * at any position within the file. */
    SysFseek(f->File,0);

    if (fl >= 2)
	return (MtxObject_t *)MatRead(f);
    else if (fl == -1)
	return (MtxObject_t *)PermRead(f);
    else if (fl == -7)
	return (MtxObject_t *)SMatRead(f);
    
    MTX_ERROR1("Unknown object type %ld found in file",fl);
    return NULL;
}




/*{{{ --------------------------------------------------------------------------
!section obj
!function XLoad "Load a matrix or permutation"
!arg fn
    File name.
!return
    Pointer to the matrix or permutation, |NULL| on error.
!description
    This function reads a matrix or permutation from a file.
!seealso MatLoad PermLoad
-------------------------------------------------------------------------- }}}*/

void *XLoad(const char *fn)	/* TODO:OBSOLETE */
{
    return MtxLoad(fn);
}


MtxObject_t *MtxLoad(const char *file_name)
{
    MtxFile_t *f;
    void *x;

    f = MfOpen(file_name);
    if (f == NULL) 
    {
	MTX_ERROR1("Cannot open %s: %S",file_name);
	return NULL;
    }
    x = MtxRead(f);
    if (x == NULL)
	MTX_ERROR1("Error reading %s",file_name);
    return x;
}




/*{{{ --------------------------------------------------------------------------
!function XSave "Save a matrix or permutation"
!arg a
    Matrix or permutation.
!arg fn
    File name.
!return
    $0$ on success, $-1$ on error.
!description
    This function writes a matrix or permutation to a file. If a file with
    the given name exists, it is destroyed.
!seealso 
-------------------------------------------------------------------------- }}}*/

int XSave(void *a, const char *fn)	/* TODO: OBSOLETE */
{
    return MtxSave((const MtxObject_t *)a,fn);
}


int MtxSave(const MtxObject_t *obj, const char *file_name)
{
    switch (obj->Type)
    {
	case MTX_MATRIX:
	    return MatSave((const Matrix_t*)obj,file_name);
	case MTX_SPARSE_MATRIX:
	    return SMatSave((const SparseMatrix_t*)obj,file_name);
	case MTX_PERMUTATION:
	    return PermSave((const Perm_t *)obj,file_name);
	case MTX_REPRESENTATION:
	    return RepSave((const Rep_t *)obj,file_name);
	case MTX_POLYNOMIAL:
	    return PolSave((const Poly_t *)obj,file_name);
    }
    MTX_ERROR1("Unknown object type %d",obj->Type);
    return -1;
}




/*{{{ --------------------------------------------------------------------------
!function XMul "Multiply matrices or permutations"
!arg a
    First matrix or permutation.
!arg b
    Second matrix or permutation.
!description
    This function multiplies a matrix or permutation by a sencond matrix
    or permutation. Note: both objects must be of the same type.
!seealso MatMul PermMul
-------------------------------------------------------------------------- }}}*/

void XMul(void *a, void *b)

{
    if (IS_MATRIX(a))
	MatMul((Matrix_t *)a,(Matrix_t *)b);
    else
	PermMul((Perm_t *)a,(Perm_t *)b);
}



/*{{{ --------------------------------------------------------------------------
!function XOrder "Order of a matrix or permutation"
!arg a
    The matrix or permutation.
!description
    This function calculates the order of a matrix or permutation.
!seealso MatOrder PermOrder
-------------------------------------------------------------------------- }}}*/

long XOrder(void *a)

{
    if (IS_MATRIX(a))
	return MatOrder((Matrix_t *)a);
    else
	return PermOrder((Perm_t *)a);
}



/*{{{ --------------------------------------------------------------------------
!function XIsCompatible "Check object compatibility"
!arg a
    First matrix or permutation.
!arg b
    Second matrix or permutation.
!description
    This function checks if two objects are compatible for |XMul()|, i.e., 
    if they are of the same type (matrix or permutation) and have the same 
    attributes.
!seealso XMul
-------------------------------------------------------------------------- }}}*/

int XIsCompatible(void *a, void *b)

{
    if (IS_MATRIX(a))
    {
	Matrix_t *am = (Matrix_t *) a;
	Matrix_t *bm = (Matrix_t *) b;
	return IS_MATRIX(b) && am->Field == bm->Field && am->Nor == bm->Nor 
	    && am->Noc == bm->Noc;
    }
    else
    {
	Perm_t *ap = (Perm_t *) a;
	Perm_t *bp = (Perm_t *) b;
	return !IS_MATRIX(b) && ap->Degree == bp->Degree;
    }
}


/*{{{ --------------------------------------------------------------------------
!function XDup "Duplicate a matrix or permutation"
!arg a
    The matrix or permutation.
!return
    Pointer to a copy of |a|, or |NULL| on error.
!description
    This function creates a copy of a matrix or permutation.
!seealso 
-------------------------------------------------------------------------- }}}*/

void *XDup(void *a)
{
    return IS_MATRIX(a) ? (void *) MatDup(a) : (void *)PermDup(a);
}



/*{{{ --------------------------------------------------------------------------
!function XInverse "Inverse of a matrix or permutation"
!arg a
    The matrix or permutation.
!return
    Pointer to a the inverse of |a|, or |NULL| on error.
!description
    This function inverts a matrix or permutation.
!seealso 
-------------------------------------------------------------------------- }}}*/

void *XInverse(void *a)

{
    return IS_MATRIX(a) ? (void *) MatInverse(a) : (void *)PermInverse(a);
}


/*{{{ --------------------------------------------------------------------------
!function XFree "Freea matrix or permutation"
!arg a
    The matrix or permutation.
!description
    This function destroys a matrix or permutation.
!seealso 
-------------------------------------------------------------------------- }}}*/

void XFree(void *a)	/* TODO: obsolete, replace with MtxFree() */
{
    if (IS_MATRIX(a))
	MatFree((Matrix_t *)a);
    else
	PermFree((Perm_t *)a);
}




int MtxFree(MtxObject_t *obj)
{
    switch (obj->Type)
    {
	case MTX_MATRIX:         MatFree((Matrix_t*) obj); return 0;
	case MTX_PERMUTATION:    PermFree((Perm_t*) obj); return 0;
	case MTX_SPARSE_MATRIX:  SMatFree((SparseMatrix_t *) obj); return 0;
	case MTX_REPRESENTATION: RepFree((Rep_t*)obj); return 0;
	case MTX_POLYNOMIAL:     PolFree((Poly_t *)obj); return 0;
    }
    MTX_ERROR1("Unknown object type %d",obj->Type);
    return -1;
}




/*{{{ --------------------------------------------------------------------------
!function XPower "Power of a matrix or permutation"
!arg a
    The matrix or permutation.
!arg n
    The power.
!return
    |n|-th power of |a|, or |NULL| on error.
!description
    This function calculates the $n$-th power of a matrix or permutation.
    $n$ may be negative.
!seealso 
-------------------------------------------------------------------------- }}}*/

void *XPower(void *a, int n)	/* TODO: Use MtxObject_t */

{
    void *b;
    int neg = 0;

    if (n < 0)
    {
	a = XInverse(a);
	if (a == NULL)
	    return NULL;
	n = -n;
	neg = 1;
    }

    if (IS_MATRIX(a))
	b = (void *) MatPower((Matrix_t *) a,n);
    else
	b = (void *) PermPower((Perm_t *) a,n);
    if (neg)
	XFree(a);
    return b;
}




int MtxIsValid(const MtxObject_t *obj)
{
    switch (obj->Type)
    {
	case MTX_MATRIX:        return MatIsValid((const Matrix_t*) obj);
	case MTX_PERMUTATION:   return PermIsValid((const Perm_t*) obj);
	case MTX_SPARSE_MATRIX: return SMatIsValid((const SparseMatrix_t *) obj);
	case MTX_REPRESENTATION: return RepIsValid((const Rep_t *) obj);
	case MTX_POLYNOMIAL:    return PolIsValid((const Poly_t *) obj);
    }
    MTX_ERROR1("Unknown object type %d",obj->Type);
    return 0;
}


MtxObject_t *MtxInverse(const MtxObject_t *obj)
{
    switch (obj->Type)
    {
	case MTX_MATRIX:
	    return (MtxObject_t *) MatInverse((const Matrix_t*) obj);
	case MTX_PERMUTATION:
	    return (MtxObject_t *) PermInverse((const Perm_t*) obj);
	case MTX_SPARSE_MATRIX: 
	case MTX_REPRESENTATION:
	case MTX_POLYNOMIAL:
	    MTX_ERROR1("Inverse not implemented for type %d",obj->Type);
	    break;
    }
    MTX_ERROR1("Unknown object type %d",obj->Type);
    return NULL;
}




MtxObject_t *MtxTransposed(const MtxObject_t *obj)
{
    switch (obj->Type)
    {
	case MTX_MATRIX:
	    return (MtxObject_t *) MatTransposed((const Matrix_t*) obj);
	case MTX_PERMUTATION:   
	case MTX_SPARSE_MATRIX: 
	case MTX_REPRESENTATION:
	case MTX_POLYNOMIAL:
	    MTX_ERROR1("Transposed not implemented for type %d",obj->Type);
	    break;
    }
    MTX_ERROR1("Unknown object type %d",obj->Type);
    return NULL;
}




int MtxCompare(const MtxObject_t *a, const MtxObject_t *b)
{
    /* Compare is currently implemented only if <a> and <b>
     * have the same types.
     * */
    MTX_VERIFY(a->Type == b->Type);

    switch (a->Type)
    {
        case MTX_MATRIX:
            return MatCompare((const Matrix_t*) a,(const Matrix_t*) b);
        case MTX_PERMUTATION:
            return PermCompare((const Perm_t*) a,(const Perm_t*) b);
        case MTX_POLYNOMIAL:
            return PolCompare((const Poly_t*) a,(const Poly_t*) b);
        case MTX_SPARSE_MATRIX:
        case MTX_REPRESENTATION:
            MTX_ERROR1("Transposed not implemented for type %d",a->Type);
            break;
    }
    MTX_ERROR1("Unknown object type %d",a->Type);
    return -1;
}




