/* ============================= C MeatAxe ==================================
   File:        $Id: permread.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Read a permutation.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



void Perm_ConvertOld(long *data, int len)

{
    int i;

    /* Check if this is an old-style permutation. We check
       if 0 in included (= news tyle) or not (=old style).
       ---------------------------------------------------- */
    for (i = 0; i < len; ++i)
    {
	if (data[i] == 0)	/* New format: nothing to do */
	    return;
    }

    /* Convert.
       --------  */
    for (i = 0; i < len; ++i)
	--data[i];
}






/*{{{ --------------------------------------------------------------------------
!section obj.perm
!function PermRead "Read a permutation"
!arg f
    File to read from.
!return
    Pointer to the permutation read from |f|, or |NULL| on error.
!description
    This function reads a permutation from a file. |f| must be a pointer to an
    open file with read permission. 
    If a permutation was successfully read, the function returns a pointer to 
    a newly created |Perm_t| object. The caller is responsible for deleting 
    this object as soon as the permutation is no longer needed.
!seealso PermLoad
-------------------------------------------------------------------------- }}}*/

Perm_t *PermRead(MtxFile_t *f)
{
    Perm_t *p;

    if (MfReadHeader(f) != 0)
	return NULL;
    if (f->Type != MTX_PERMUTATION) 
    {
	MTX_ERROR1("%E", MTX_ERR_NOTPERM);
	return NULL;
    }
    p = PermAlloc(f->Degree);
    if (p == NULL) 
	return NULL;
    if (MfReadLong(f,p->Data,p->Degree) != p->Degree)
    {
	PermFree(p);
	MTX_ERROR("Cannot read permutation data");
	return NULL;
    }
    Perm_ConvertOld(p->Data,p->Degree);
    if (!PermIsValid(p))
	return NULL;
    return p;
}


/*{{{ --------------------------------------------------------------------------
!function PermLoad "Read a permutation"
!arg fn
    File name.
!return
    Pointer to the permutation read from the file, or |NULL| on error.
!description
    This function opens a file, reads a single permutation, and closes the 
    file. The return value is a pointer to the permutation or \verb"NULL" on 
    error. If the file contains more than one permutation, only the first one 
    is read.
    If a permutation was successfully read, the function returns a pointer to 
    a newly created |Perm_t| object. The caller is responsible for deleting 
    this object as soon as the permutation is no longer needed.
!seealso PermRead
-------------------------------------------------------------------------- }}}*/

Perm_t *PermLoad(const char *fn)
{
    MtxFile_t *f;
    Perm_t *p;

    if ((f = MfOpen(fn)) == NULL)
	return NULL;
    p = PermRead(f);
    MfClose(f);
    if (p == NULL) 
    {
	MTX_ERROR1("Cannot read permutation from %s",fn);
	return NULL;
    }
    return p;
}





/* Called from MfReadHeader() */

int PermEvalHeader(MtxFile_t *f, long hdr[3])
{
    f->Type = MTX_PERMUTATION;
    f->Degree = (int) hdr[1];
    f->DataSize = f->Degree * 4;
    if (f->Degree < 0)
    {
	MTX_ERROR1("%s: Invalid permutation header",f->FileName);
	return -1;
    }
    f->Field = -1;		/* for compatibility */
    f->Nor = f->Degree;		/* for compatibility */
    f->Noc = (int) hdr[2];	/* for compatibility (# of perms) */
    return 0;
}


