/* ============================= C MeatAxe ==================================
   File:        $Id: polread.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Read a polynomial from a file.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO



/* -----------------------------------------------------------------
   mktmp() - Allocate temporary workspace
   ----------------------------------------------------------------- */

static long tmpfl = 0;
static long tmpdeg = 0;
static PTR tmpvec = NULL;

static void mktmp(long fl, long deg)

{
    FfSetField(fl);
    if (deg > 0) FfSetNoc(deg+1);
    if (tmpfl != fl || tmpdeg < deg)
    {
	if (tmpvec != NULL) SysFree(tmpvec);
	tmpvec = FfAlloc(1);
	tmpdeg = deg;
	tmpfl = fl;
    }
}



/*{{{ --------------------------------------------------------------------------
!section obj.poly
!function PolRead "Read a polynomial from a file"
!arg f
    File to read from.
!return
    Pointer to the plynomial, or |NULL| on error.
!description
    This function reads a polynomial from a file. If successful, the return 
    value is a pointer to a new |Poly_t| object. The caller is responsible
    for deleting this polynomial with |PolFree()| when it is no longer needed.
!seealso PolLoad
-------------------------------------------------------------------------- }}}*/

Poly_t *PolRead(MtxFile_t *f)
{
    Poly_t *p;
    
    if (MfReadHeader(f) != 0)
	return NULL;
    if (f->Type != MTX_POLYNOMIAL)
    {
	MTX_ERROR1("No polynomial (type=%d)",f->Field);
	return NULL;
    }
    mktmp(f->Field,f->Degree);
    if ((p = PolAlloc(f->Field,f->Degree)) == NULL)
    {
	MTX_ERROR("Cannot create polynomial");
	return NULL;
    }
    if (p->Degree > 0)
    {
	int i;
	if (MfReadRows(f,tmpvec,1) != 1)
	{
	    PolFree(p);
	    MTX_ERROR("Cannot read data");
	    return NULL;
	}
	for (i = 0; i <= p->Degree; ++i)
	    p->Data[i] = FfExtract(tmpvec,i);
    }
    return p;
}





/*{{{ --------------------------------------------------------------------------
!section obj.poly
!function PolLoad "Read a polynomial"
!synopsis 
    Poly_t *PolLoad(const char *fn);
!arg fn
    File name.
!return
    Pointer to the polynomial read from the file, or |NULL| on error.
!description
    This function opens a file, reads a single polynomial, and closes the 
    file. The return value is a pointer to the polynomial or \verb"NULL" on 
    error. If the file contains more than one polynomial, only the first one 
    is read.

    If a polynomial was successfully read, the function returns a pointer to 
    a newly created |Poly_t| object. The caller is responsible for deleting 
    this object as soon as it no longer needed.
!seealso PolRead
-------------------------------------------------------------------------- }}}*/

Poly_t *PolLoad(const char *fn)
{
    MtxFile_t *f;
    Poly_t *p;

    if ((f = MfOpen(fn)) == NULL)
	return NULL;
    p = PolRead(f);
    MfClose(f);
    if (p == NULL) 
    {
	MTX_ERROR1("Cannot read polynomial from %s",fn);
	return NULL;
    }
    return p;
}




/* Called from MfReadHeader() */

int PolEvalHeader(MtxFile_t *f, long hdr[3])
{
    f->Type = MTX_POLYNOMIAL;
    f->Field = (int) hdr[1];
    f->Degree = (int) hdr[2];
    if (f->Degree < 0  || f->Field < 2)
    {
	MTX_ERROR1("%s: Invalid polynomial header",f->FileName);
	return -1;
    }
    f->Nor = (int) hdr[1];	/* for compatibility */
    f->Noc = (int) hdr[2];	/* for compatibility */
    FfSetField(f->Field);
    FfSetNoc(f->Degree);
    f->DataSize = FfTrueRowSize(f->Degree + 1);
    return 0;
}


