/* ============================= C MeatAxe ==================================
   File:        $Id: mrcore.c,v 1.2 2000/09/02 15:32:32 mringe Exp $
   Comment:     Basic functin s for represenations.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO


static int GetFieldAndDimension(const MtxObject_t *gen, int *fld, int *dim)
{
    *fld = *dim = -1;
	switch (gen->Type)
	{
	    case MTX_MATRIX:
		*fld = ((Matrix_t *)gen)->Field;
		*dim = ((Matrix_t *)gen)->Nor;
		if (((Matrix_t *)gen)->Noc != *dim)
		{
		    MTX_ERROR("Matrix not square");
		    return -1;
		}
		break;
	    case MTX_SPARSE_MATRIX:
		*fld = ((SparseMatrix_t *)gen)->Field;
		*dim = ((SparseMatrix_t *)gen)->Nor;
		if (((SparseMatrix_t *)gen)->Noc != *dim)
		{
		    MTX_ERROR("Sparse matrix not square");
		    return -1;
		}
		break;
	    case MTX_PERMUTATION:
		*fld = -1;
		*dim = ((Perm_t *)gen)->Degree;
	        break;
	    default:
	    	MTX_ERROR1("Invalid generator type %d",gen->Type);
		return -1;
	}
    return 0;
}

   


static int GensAreValid(int ngen, MtxObject_t **gen)
{
    int i;
    int field0=0, dim0=0;

    if (ngen < 0)
    {
	MTX_ERROR1("ngen: %E",MTX_ERR_BADARG);
	return 0;
    }
    if (ngen > 0 && gen == NULL)
    {
	MTX_ERROR1("gen == NULL: %E",MTX_ERR_BADARG);
	return 0;
    }
    for (i = 0; i < ngen; ++i)
    {
	int field = 0, dim = 0;

	if (   !MtxIsValid(gen[i])
 	    || GetFieldAndDimension(gen[i],&field,&dim) != 0
	   )
	{
	    MTX_ERROR1("gen[%d] invalid",i);
	    return 0;
	}
	if (i == 0)
	{
	    field0 = field;
	    dim0 = dim;
	}
	else if (field != field0 || dim != dim0)
	{
	    MTX_ERROR2("gen[0] and gen[%d]: %E",i,MTX_ERR_INCOMPAT);
	    return 0;
	}
    }
    return 1;
}


/*{{{ --------------------------------------------------------------------------
!section obj.matrep
!function MrIsValid "Check a representation"
!arg rep
    Pointer to the representation.
!return
    1 if |rep| points to a valid representation, 0 otherwise.
!description
    This function checks if the argument |rep| is a pointer to a valid
    representation. If the representation is o.k., the function 
    returns 1. Otherwise, an error is signaled and, if the error handler 
    does not terminate the program, the function returns 0.
!seealso Rep_t
-------------------------------------------------------------------------- }}}*/

int RepIsValid(const Rep_t *rep)
{
    if (rep == NULL)
    {
	MTX_ERROR("NULL representation");
	return 0;
    }
    if (rep->Type != MTX_REPRESENTATION)
    {
	MTX_ERROR1("Invalid representation (type=%d)",rep->Type);
	return 0;
    }
    if (!GensAreValid(rep->NGen,rep->Gen))
    {
	MTX_ERROR("Invalid generators");
	return 0;
    }
    return 1;
}




/*{{{ --------------------------------------------------------------------------
!function MrAlloc "Create a new representation"
!arg ngen
    Number of generators in |gen|.
!arg gen
    List of generators.
!return
    Pointer to the new representation or |NULL| on error.
!description
    This function creates a new representation. Matrices representing
    the generators must be passed in |gen|. The matrices in |gen| must all be 
    square, over the same field, and with the same dimensions.

    Only references to the matrices 
    are stored in the |Rep_t| structure. Consequently, the application 
    must not modify or destroy the matrices after calling |MrAlloc()|. They
    will be destroyed automatically when |MrFree()| is called to destroy the
    representation.
!seealso MrFree Rep_t
-------------------------------------------------------------------------- }}}*/

Rep_t *RepAlloc(int ngen, MtxObject_t **gen)
{
    Rep_t *rep;
    int i;

    if (!GensAreValid(ngen,gen))
	return NULL;

    /* Allocate a new Rep_t structure
       ------------------------------ */
    rep = ALLOC(Rep_t);
    if (rep == NULL)
    {
	MTX_ERROR("Cannot allocate Rep_t structure");
	return NULL;
    }
    memset(rep,0,sizeof(Rep_t));
    rep->Gen = NALLOC(MtxObject_t *,ngen);
    if (rep->Gen == NULL)
    {
	MTX_ERROR("Cannot allocate generator list");
	SysFree(rep);
	return NULL;
    }

    /* Copy generators
       --------------- */
    rep->NGen = ngen;
    for (i = 0; i < ngen; ++i)
	rep->Gen[i] = gen[i];

    rep->Type = MTX_REPRESENTATION;
    return rep;
}




/*{{{ --------------------------------------------------------------------------
!function RepFree "Delete a representation"
!arg rep
    Pointer to the representation.
!return
    $0$ on success, $-1$ on error.
!description
    This function frees a representation which has been created by 
    |RepAlloc()|. This implies freeing the internal data buffers, all
    Generators, and finally the |Rep_t| structure itself.
-------------------------------------------------------------------------- }}}*/

int RepFree(Rep_t *rep)
{
    int i;
    if (!RepIsValid(rep))
    {
	MTX_ERROR1("%E",MTX_ERR_BADARG);
	return -1;
    }
    for (i = 0; i < rep->NGen; ++i)
        MtxFree(rep->Gen[i]);
    memset(rep->Gen,0,sizeof(MtxObject_t *) * rep->NGen);
    SysFree(rep->Gen);
    memset(rep,0,sizeof(*rep));
    SysFree(rep);
    return 0;
}


int MtxGetNoc(const MtxObject_t *gen)
{
    int repf, dim;
    if (GetFieldAndDimension(gen,&repf,&dim) != 0)
	return 0;
    return dim;
}


int MtxCanMap(const MtxObject_t *gen, int field, int noc)
{
    int repf, dim;
    if (GetFieldAndDimension(gen,&repf,&dim) != 0)
	return 0;
    if (repf > 0 && repf != field)
	return 0;
    if (dim > 0 && noc != dim)
	return 0;
    return 1;
}

int RepCanMap(const Rep_t *rep, int field, int noc)
{
    return rep->NGen == 0 || MtxCanMap(rep->Gen[0],field,noc);
}


int RepAddGenerator(Rep_t *rep, MtxObject_t *gen)
{
    MtxObject_t **new_gen;
    int fld, dim;

    /* Check arguments
       --------------- */
    if (!RepIsValid(rep))
    {
	MTX_ERROR1("rep: %E",MTX_ERR_BADARG);
	return -1;
    }
    if (!MtxIsValid(gen))
    {
	MTX_ERROR("Invalid generator");
	return -1;
    }

    /* Check if <gen> is acceptable as a generator.
       -------------------------------------------- */
    if (GetFieldAndDimension(gen,&fld,&dim) != 0)
    {
	MTX_ERROR("Invalid generator");
	return -1;
    }
    if (rep->NGen > 0)
    {
	int fld0, dim0;
	GetFieldAndDimension(rep->Gen[0],&fld0,&dim0);
	if (fld0 != fld || dim0 != dim)
	{
	    MTX_ERROR("Generators are incompatible");
	    return -1;
	}
    }
	
    /* Add the generator.
       ------------------ */
    new_gen = NREALLOC(rep->Gen,MtxObject_t *,rep->NGen + 1);
    if (new_gen == NULL)
    {
	MTX_ERROR("Cannot extend generator list");
	return -1;
    }
    new_gen[rep->NGen] = gen;
    rep->Gen = new_gen;
    ++rep->NGen;

    return 0;
}




Rep_t *RepLoad(const char *basename, int ngen)
{
    char *fn;
    int ext_format;	    /* '%d' found in <basename> */
    Rep_t *mr;
    int i;

    /* Make a copy of the basename and reserve extra bytes for the extension.
       ---------------------------------------------------------------------- */
    fn = SysMalloc(strlen(basename) + 10);
    if (fn == NULL)
    {
	MTX_ERROR("Cannot allocate buffer");
	return NULL;
    }

    /* Allocate the Representation.
       ---------------------------- */
    mr = RepAlloc(0,NULL);
    if (mr == NULL)
    {
	MTX_ERROR("Cannot allocate representation");
	SysFree(fn);
	return NULL;
    }

    /* Read the generators.
       -------------------- */
    ext_format = strstr(basename,"%d") != NULL;
    for (i = 0; i < ngen; ++i)
    {
	MtxObject_t *gen;
	if (ext_format)
	    sprintf(fn,basename,i+1);
	else
	    sprintf(fn,"%s.%d",basename,i+1);
	if ((gen = MtxLoad(fn)) == NULL || RepAddGenerator(mr,gen) != 0)
	{
	    MTX_ERROR("Cannot load generator");
	    RepFree(mr);
	    SysFree(fn);
	    return NULL;
	}
    }

    SysFree(fn);
    return mr;
}








int RepSave(const Rep_t *rep, const char *basename)

{
    char *fn;
    int ext_format;	    /* '%d' found in <basename> */
    int i;

    /* Make a copy of the basename an reserve extra bytes for the extension
       -------------------------------------------------------------------- */
    fn = SysMalloc(strlen(basename) + 10);
    if (fn == NULL)
    {
	MTX_ERROR("Cannot allocate buffer");
	return -1;
    }

    /* Write the generators.
       --------------------- */
    ext_format = strstr(basename,"%d") != NULL;
    for (i = 0; i < rep->NGen; ++i)
    {
	if (ext_format)
	    sprintf(fn,basename,i+1);
	else
	    sprintf(fn,"%s.%d",basename,i+1);
	if (MtxSave(rep->Gen[i],fn) != 0)
	{
	    MTX_ERROR1("Error writing generator %d",i + 1);
	    break;
	}
    }

    /* Clean up.
       --------- */
    SysFree(fn);
    return i >= rep->NGen ? 0 : -1;
}

