/* ============================= C MeatAxe ==================================
   File:        $Id: smatcore.c,v 1.2 2003/04/09 18:12:39 hoffmant Exp $
   Comment:     Basic matrix functions.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   Modified by Tom Hoffman, Mathematics Department, University of
   Arizona, USA <hoffmant@math.arizona.edu>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO


   
   

/*{{{ --------------------------------------------------------------------------
!definesection obj.smat
\index{SparseMatrix_t@{\tt SparseMatrix\protect\_t} data type}
In the {\MeatAxe}, row sparse matrices over finite fields are represented by 
|SparseMatrix_t| structures. Besides the marks, each matrix carries the field 
order, the number of rows and the number of columns. There is no global field 
order or row length as at the kernel layer.

Sparse matrices can be created in many ways, including (i) calling 
|SMatAlloc()|, (ii) making a copy of an existing sparse matrix with |MatDup()|,
and (iii) by reading a sparse matrix from a data file with |SMatRead()| or 
|SMatLoad()|. The application is responsible for releasing sparse matrices 
which are no longer needed. memory.  There is only one possibility of deleting
a sparse matrix: calling |SMatFree()|.
-------------------------------------------------------------------------- }}}*/


/*{{{ --------------------------------------------------------------------------
!section obj.smat
!function SMatIsValid "Check a sparse matrix"
!synopsis 
    int SMatIsValid(const SparseMatrix_t *smat);
!arg smat
    Pointer to the sparse matrix.
!return
    1 if |smat| points to a valid sparse matrix, 0 otherwise.
!description
    This function checks if the argument |smat| is a pointer to a valid
    sparse matrix. If the sparse matrix is o.k., the function returns 1. 
    Otherwise, an error is signaled and, if the error handler does not 
    terminate the program, the function returns 0.
!seealso SparseMatrix_t
-------------------------------------------------------------------------- }}}*/

int SMatIsValid(const SparseMatrix_t *smat)
{
    if (smat == NULL)
    {
	MTX_ERROR("NULL sparse matrix");
	return 0;
    }
    if (smat->Type != MTX_SPARSE_MATRIX || smat->Field < 2 || smat->Nor < 0 || 
	smat->Noc < 0)
    {
	MTX_ERROR3("Invalid sparse matrix (field=%d, nor=%d, noc=%d)",
            smat->Field,smat->Nor,smat->Noc);
	return 0;
    }
    return 1;

}




/*{{{ --------------------------------------------------------------------------
!function SMatAlloc "Create a new sparse matrix"
!synopsis 
    SparseMatrix_t *SMatAlloc(int field, int nor, int noc);
!arg field
    Field order.
!arg nor
    Number of rows.
!arg noc
    Number of columns.
!return
    Pointer to the new sparse matrix or |NULL| on error.
!description
    This function creates a new sparse matrix with |nor| rows and |noc|
    columns over GF(\verb"field"). The return value is a pointer to the 
    allocated |SparseMatrix_t| structure, or |NULL| if there is not enough 
    memory. 

    To destroy a sparse matrix, use |SMatFree()|, {\em not} \verb"SysFree()".
!seealso SMatFree SparseMatrix_t
-------------------------------------------------------------------------- }}}*/

SparseMatrix_t *SMatAlloc(int field, int nor, int noc, int nonzero)

{
    SparseMatrix_t *m;

    MTX_VERIFY(field >= 2);
    MTX_VERIFY(nor >= 0);
    MTX_VERIFY(noc >= 0);

    /* Allocate a new Sparse Matrix_t structure
       ---------------------------------------- */
    m = ALLOC(SparseMatrix_t);
    if (m == NULL)
    {
	MTX_ERROR("Cannot allocate SparseMatrix_t structure");
	return NULL;
    }

    /* Initialize the data structure
       ----------------------------- */
    if (FfSetField(field) != 0)
    {
	MTX_ERROR1("Cannot select field GF(%d)",field);
	SysFree(m);
	return NULL;
    }
    FfSetNoc(nonzero);
    m->Type = MTX_SPARSE_MATRIX;
    m->Field = field;
    m->Nor = nor;
    m->Noc = noc;
    m->NonZero = nonzero;
    m->Data = FfAlloc(nor);
    m->Position = NALLOC(long,nor*nonzero);
    m->RowSize = FfCurrentRowSize;
    if (m->Data == NULL || m->Position == NULL)
    {
	SysFree(m);
	MTX_ERROR("Cannot allocate sparse matrix data");
	return NULL;
    }
    return m;
}




/*{{{ --------------------------------------------------------------------------
!function SMatGetPtr "Pointer to a row of a sparse matrix"
!arg mat
    Pointer to the sparse matrix.
!arg row
    Row index.
!return
    Pointer to the selected row, or |NULL| on error.
!description
    This function returns a pointer to the row number |row| of a sparse matrix.
    Row numbers start from 0. The current row size is not changed.
!seealso
-------------------------------------------------------------------------- }}}*/

PTR SMatGetPtr(const SparseMatrix_t *smat, int row)

{
#ifdef DEBUG
    if (!SMatIsValid(smat))
	return NULL;
#ifdef PARANOID
    if (row < 0 || row >= smat->Nor)
#else
    if (row < 0 || row > smat->Nor + 5)
#endif
    {
	MTX_ERROR2("row=%d: %E",row,MTX_ERR_BADARG);
	return NULL;
    }
#endif
    return (PTR)((char *) smat->Data + smat->RowSize * row);
}



/*{{{ --------------------------------------------------------------------------
!function SMatFree "Delete a sparse matrix"
!synopsis 
    int SMatFree(SparseMatrix_t *m);
!arg mat
    Pointer to the sparse matrix.
!return
    $0$ on success, $-1$ on error.
!description
    This function frees a sparse matrix which has beed created by |SMatAlloc()|.
    This implies freeing the internal data buffers as well as the 
    |SparseMatrix_t| structure itself.
!seealso SMatAlloc SparseMatrix_t
-------------------------------------------------------------------------- }}}*/

int SMatFree(SparseMatrix_t *smat)

{
    if (!SMatIsValid(smat))
	return -1;
    if (smat->Data != NULL)
	SysFree(smat->Data);
    memset(smat,0,sizeof(SparseMatrix_t));
    SysFree(smat);
    return 0;
}
















