/* ============================= C MeatAxe ==================================
   File:        $Id: smatread.c,v 1.2 2003/04/09 18:12:39 hoffmant Exp $
   Comment:     Read a matrix from a file.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   Modified by Tom Hoffman, Mathematics Department, University of
   Arizona, USA <hoffmant@math.arizona.edu>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <stdlib.h>

MTX_DEFINE_FILE_INFO



/*{{{ --------------------------------------------------------------------------
!section obj.mat
!function SMatRead "Read a sparse matrix from a file"
!synopsis 
    SparseMatrix_t *SMatRead(FILE *f);
!arg f
    File to read from.
!return
    Pointer to the sparse matrix, or |NULL| on error.
!description
    This function reads a sparse matrix from a file.
!seealso MatLoad
-------------------------------------------------------------------------- }}}*/

SparseMatrix_t *SMatRead(MtxFile_t *f)
{
    SparseMatrix_t *m;
    long i;

    if (MfReadHeader(f) != 0) 
	return NULL;
    if (f->Type != MTX_SPARSE_MATRIX) 
    {
	MTX_ERROR1("%E",MTX_ERR_NOTMATRIX);
	return NULL;
    }
    m = SMatAlloc(f->Field,f->Nor,f->Noc,f->NonZero);
    if (m == NULL) 
	return NULL;
    for (i=0; i < m->Nor; i++)
      {
	if (MfReadRows(f,m->Data+(i*m->RowSize),1) != 1)
	  {
	    SMatFree(m);
	    return NULL;
	  }	
	if (MfReadLong(f,m->Position + i * m->NonZero, m->NonZero) != m->NonZero)
	  {
	    SMatFree(m);
	    return NULL;
	  }
      }
    return m;
}



/*{{{ --------------------------------------------------------------------------
!function SMatLoad "Read a sparse matrix from a file"
!synopsis 
    SparseMatrix_t *SMatLoad(const char *fn);
!arg fn
    File name.
!return
    Pointer to the sparse matrix, or |NULL| on error.
!description
    This function opens a file, reads a single sparse matrix, and closes the 
    file.

    To read more than one matrix from a file, use |SMatRead()|.
!seealso SMatRead
-------------------------------------------------------------------------- }}}*/

SparseMatrix_t *SMatLoad(const char *fn)
{
    MtxFile_t *f;
    SparseMatrix_t *m;

    if ((f = MfOpen(fn)) == NULL)
	return NULL;
    m = SMatRead(f);
    MfClose(f);
    return m;
}




/* Called from MfReadHeader() */

int SMatEvalHeader(MtxFile_t *f, long hdr[3])
{
    long hdr2[2];
    /* Read two more header entries */
    if (SysReadLong(f->File,hdr2,2) != 2)
    {
	MTX_ERROR1("%s: Error reading file header",f->FileName);
	return -1;
    }
    f->Type = MTX_SPARSE_MATRIX;
    f->Field = (int) hdr2[0];
    f->Nor = (int) hdr[1];
    f->Noc = (int) hdr[2];
    f->NonZero = (int) hdr2[1];
    FfSetField(f->Field);
    FfSetNoc(f->Noc);
    f->DataSize = f->Noc * (FfTrueRowSize(f->NonZero) + 4 * f->NonZero);
    if (f->Nor < 0 || f->Noc < 0 || f->Field > 256 || f->NonZero > f->Noc)
    {
	MTX_ERROR1("%s: Invalid sparse matrix header",f->FileName);
	return -1;
    }
    return 0;
}


    

