/* ============================= C MeatAxe ==================================
   File:        $Id: sparse2mat.c,v 1.2 2003/04/09 18:12:39 hoffmant Exp $
   Comment:     Convert a sparse matrix into a matrix.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   Modified by Tom Hoffman, Mathematics Department, University of
   Arizona, USA <hoffmant@math.arizona.edu>
   This program is free software; see the file COPYING for details.
   ========================================================================== */


#include "meataxe.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>




/* --------------------------------------------------------------------------
   Global data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO

static MtxApplicationInfo_t AppInfo = { 
"sparse2mat", "Convert a sparse format matrix into a MeatAxe format", 
"SYNTAX\n"
"    sparse2mat <A> <Result>\n"
"\n"
"ARGUMENTS\n"
"    <A> .................... sparse matrix to be converted\n"
"    <Result> ............... resulting matrix\n"
"\n"
};


static MtxApplication_t *App = NULL;
static const char *aname, *bname;           /* File names */
static MtxFile_t *afile = NULL; 
static MtxFile_t *bfile = NULL;		    /* File handles */


void Convert(void)

{
    PTR m1, m2;
    long *row;
    FEL f1;
    long loop1, j1;

    row = NALLOC(long,afile->NonZero);
    if ((bfile = MfCreateAndWriteHeader(bname,
	afile->Field,afile->Nor,afile->Noc)) == NULL)
        return;

    /* Input sparse matrix */
    FfSetField(afile->Field);
    FfSetNoc(afile->NonZero);
    m1 = FfAlloc(1);

    /* Output matrix */
    FfSetNoc(afile->Noc);
    m2 = FfAlloc(1);

    MESSAGE(0,("matrix field=%d rows=%d cols=%d\n",afile->Field,afile->Nor,afile->Noc));
    for (loop1 = 1; loop1 <= afile->Nor; ++loop1)
    {   
        FfSetNoc(afile->NonZero);
        if (MfReadRows(afile,m1,1) != 1)
        {
            MTX_ERROR1("Cannot read %s",afile);
            return;
        }
        MfReadLong(afile,row,afile->NonZero);
        for (j1 = 0; j1 < afile->NonZero; ++j1)
        {
	  FfSetNoc(afile->NonZero);
            f1 = FfExtract(m1,j1);
	    if (row[j1] != 0)
	    {
	      FfSetNoc(afile->Noc);
	      FfInsert(m2,row[j1]-1,f1);
	    }
	}
	FfSetNoc(afile->Noc);
	if (MfWriteRows(bfile,m2,1) != 1)
        {
            MTX_ERROR1("Cannot write %s",bfile);
            return;
        }
	FfMulRow(m2,FF_ZERO);
    }
}



static int Init(int argc, const char **argv)
{

    App = AppAlloc(&AppInfo,argc,argv);
    if (App == NULL)
      return -1;

    if (AppGetArguments(App,2,2) < 0)
         return -1;

    /* Command line arguments.
       ----------------------- */
    aname = App->ArgV[0];
    bname = App->ArgV[1];
    if (!strcmp(aname,bname))
    {
	MTX_ERROR("Identical file names not allowed");
	return -1;
    }

    return 0;
}


static int OpenFiles()
{
    afile = MfOpenAndReadHeader(aname);
    if (afile == NULL || afile->Type != MTX_SPARSE_MATRIX) 
	return -1;
    return 0;
}



static void Cleanup()

{
    if (afile != NULL)
	MfClose(afile);
    if (bfile != NULL)
        MfClose(bfile);
    AppFree(App);
}


int main(int argc, const char **argv)
{   
    if (Init(argc,argv) != 0)
	return -1;
    if (OpenFiles() != 0)
	return -1;

    Convert();
    Cleanup();
    return 0;
}




/*{{{ --------------------------------------------------------------------------
!section apps.general
!program sparse2mat "Convert a sparse matrix into a matrix"
!synopsis 
    sparse2mat <A> <Result>
!arg A
    Sparse matrix (input).
!arg Result
    The corresponding MeatAxe matrix (output).
!description
    This program reads in a row sparse matrix and returns a row by row MeatAxe 
    matrix.

!seealso mat2sparse
-------------------------------------------------------------------------- }}}*/

