/* ============================= C MeatAxe ==================================
   File:        $Id: zmw.c,v 1.1.1.1 2000/01/07 16:42:27 mringe Exp $
   Comment:     Make word.
   --------------------------------------------------------------------------
   (C) Copyright 1999 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"

#include <ctype.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>




/* ------------------------------------------------------------------
   Global data
   ------------------------------------------------------------------ */

MTX_DEFINE_FILE_INFO

static int opt_G = 0;
static int opt_m = 0;				/* -m */
static int NGen = -1;
static MtxObject_t *Gen[MAXGEN];
static int WordNo = -1, WordNo2 = -1;
static Rep_t *Rep = NULL;
static WgData_t *WGen = NULL;
static Poly_t *Poly = NULL;
static const char *WordFileName = NULL;
static const char *NspFileName = NULL;
static const char *VectorFileName = NULL;	/* Input vectors (with -m) */
static const char *ImageFileName = NULL;	/* Image of vectors (with -m) */
static Matrix_t *Vectors = NULL;		/* Input vectors */


static MtxApplicationInfo_t AppInfo = { 
"zmw", "Make word", 
"SYNTAX\n"
"    zmw [<Options>] <No> <Gen1> <Gen2> [<Word> [<Nsp>]]\n"
"    zmw [<Options>] -g <Ngen> <No> <Gen> [<Word> [<Nsp>]]\n"
"    zmw [<Options>] -m <No> <Gen1> <Gen2> <Vector> <Image>\n"
"\n"
"OPTIONS\n"
MTX_COMMON_OPTIONS_DESCRIPTION
"    -G ...................... GAP output\n"
"    -g ...................... Set number of generators\n"
"    -m ...................... Map vector(s) without making the word\n"
"\n"
"ARGUMENTS\n"
"    <No> .................... Word or word range (A-B), optionally followed\n"
"                              by polynomial (A/c,c,c...c)\n"
"    <Gen1>,<Gen2> ........... Generators\n"
"    <Gen>.1, <Gen>.2,... .... Generators (with -g)\n"
"    <Word> .................. Output file name\n"
"    <Nsp> ................... Null-Space file name\n"
"\n"
"FILES\n"
"    <Gen>.{1,2...} .......... I Generators (with -g)\n"
"    <Gen1>, <Gen2> .......... I Generators (without -g)\n"
"    <Word> .................. O Word\n"
"    <Nsp> ................... O Null-Space of <Word>\n"
"    <Vectors> ............... I Input vector(s)\n"
"    <Image> ................. O Image of vectors under word <No>\n"

};

static MtxApplication_t *App = NULL;





static int ReadGenerators()

{
    int ngen = NGen == -1 ? 2 : NGen;
    int i;

    for (i = 0; i < ngen; ++i)
    {
	char fn[200];
	if (NGen != -1)
	    sprintf(fn,"%s.%d",App->ArgV[1],i + 1);
	else
	    strcpy(fn,App->ArgV[1 + i]);
	if ((Gen[i] = MtxLoad(fn)) == NULL)
	    return -1;
    }

    NGen = ngen;
    if ((Rep = RepAlloc(NGen,Gen)) == NULL)
	return -1;
    if ((WGen = WgAlloc(Rep)) == NULL)
	return -1;

    return 0;
}





/* ------------------------------------------------------------------
   ParseWord() - Parse word/polynomial specification

   Examples:
        spec          WordNo   WordNo2   Poly
	------------  ------   -------   ------------
        10            10       -1        NULL
	1-100         1        100       NULL
	30/1,-1       30       -1        x-1
	30-32/1,1,1   30       31        x^2+x+1
   ------------------------------------------------------------------ */

static int ParseWord(const char *spec)

{
    if (!isdigit(*spec))
	return -1;
    WordNo = atoi(spec);
    while (isdigit(*spec)) ++spec;
    if (*spec == '-')
    {
	++spec;
	if (!isdigit(*spec))
	    return -1;
	WordNo2 = atoi(spec);
	while (isdigit(*spec)) 
	    ++spec;
    }
    if (*spec == '/')
    {
	const char *c;
	int deg;
	++spec;

	for (c = spec, deg = 0; *c != 0; ++c)
	{
	    if (*c == ',') 
		++deg;
	}
	Poly = PolAlloc(FfOrder,deg);
	while (deg >= 0)
	{
	    if (!isdigit(*spec) && (*spec != '-' || !isdigit(spec[1])))
		return -1;
	    Poly->Data[deg--] = FfFromInt(atoi(spec));
	    while (*spec == '-' || isdigit(*spec)) 
		++spec;
	    if (*spec == ',')
		++spec;
	}
    
    }
    if (*spec != 0)
	return -1;

    MESSAGE(1,("Word %d..%d, Poly=",WordNo,WordNo2));
    if (MSG1)
    {
	if (Poly != NULL)
	{
	    PolPrint(NULL,Poly);
	    printf("\n");
	}
	else
	    printf("x\n");
    }
    return 0;
}




/* ------------------------------------------------------------------
   init() - Process command line options and arguments
   ------------------------------------------------------------------ */

static int Init(int argc, const char **argv)
{
    /* Process command line options.
       ----------------------------- */
    App = AppAlloc(&AppInfo,argc,argv);
    if (App == NULL)
	return -1;
    opt_G = AppGetOption(App,"-G --gap");
    opt_m = AppGetOption(App,"-m, --map-vector");
    if (opt_G) MtxMessageLevel = -100;
    NGen = AppGetIntOption(App,"-g",-1,1,MAXGEN);

    /* Process arguments.
       ------------------ */
    if (opt_m)
    {
	int num_args = (NGen == -1) ? 5 : 4;
	if (AppGetArguments(App,num_args,num_args) < 0)
	    return -1;
	VectorFileName = App->ArgV[num_args - 2];
	ImageFileName = App->ArgV[num_args - 1];
    }
    else
    {
	int min_num_args = (NGen == -1) ? 3 : 2;
	if (AppGetArguments(App,min_num_args,min_num_args + 2) < 0)
	    return -1;
	if (App->ArgC > min_num_args)
	    WordFileName = App->ArgV[min_num_args];
	if (App->ArgC > min_num_args + 1)
	    NspFileName = App->ArgV[min_num_args + 1];
    }

    /* Other initialization.
       --------------------- */
    if (ReadGenerators() != 0)
	return -1;
    /* Must be after ReadGenerators() because the field must be known */
    if (ParseWord(App->ArgV[0]) != 0)
    {
	MTX_ERROR("Invalid word/polynomial specification");
	return -1;
    }
    if (opt_m && (Vectors = MatLoad(VectorFileName)) == NULL)
	return -1;
    if (opt_m && (WordNo2 >= 0 || Poly != NULL))
    {
	MTX_ERROR("-m cannot be used with polynomials or multiple words");
	return -1;
    }

    return 0;
}




static void Cleanup()
{
    WgFree(WGen);
    RepFree(Rep);
    AppFree(App);
}



static int MakeWord()
{
    Matrix_t *w;
    int lastword = WordNo2 >= 0 ? WordNo2 : WordNo;


    if (Poly != NULL && MSG0)
    {
	PolPrint("Using polynomial p(x)",Poly);
    }
    MESSAGE(0,("Number Nullity Word\n"));
    for (; WordNo <= lastword; ++WordNo)
    {
	w = WgMakeWord(WGen,WordNo);
	if (w == NULL)
	    return -1;
	if (Poly != NULL)
	    MatInsert_(w,Poly);
	MESSAGE(0,("%6d",WordNo));
	if (WordFileName != NULL && WordNo2 == -1)
	    MatSave(w,WordFileName);
	if (WordNo2 != -1 || NspFileName != NULL)
	{
	    Matrix_t *nsp = MatNullSpace_(w);
	    if (WordNo2 == -1)
		MatSave(nsp,NspFileName);
	    MESSAGE(0,("%8d",nsp->Nor));
	    MatFree(nsp);
	}
	else
	    MESSAGE(0,("        "));
	MESSAGE(0,(" %s\n",WgSymbolicName(WGen,WordNo)));
	MatFree(w);
    }
    return 0;
}



static int MapVectors()
{
    Matrix_t *result = WgMap(WGen,WordNo,Vectors);
    MatSave(result,ImageFileName);
    return 0;
}



/* ------------------------------------------------------------------
   main()
   ------------------------------------------------------------------ */

int main(int argc, const char **argv)
{
    int rc;

    if (Init(argc,argv) != 0)
	return 1;
    if (opt_m)
        rc = MapVectors();
    else
        rc = MakeWord();
    Cleanup();
    return rc;
}


/*{{{ --------------------------------------------------------------------------
!section apps.general
!program zmw "Make word"
!synopsis 
    zmw [<Options>] <No> <Gen> [<Word> [<Nsp>]]
    zmw [<Options>] -m <NGen> <No> <Gen> <Vector> <Image>
!arg No
    Word/polynomial specification.
!arg Gen1
    First generator.
!arg Gen2
    Second generator.
!arg Gen
    generator base name (with -g).
!arg Word
    Word (output).
!arg Nsp
    Kernel (output).
!description
    ZMW calculates an element of the algebra generated by a set of matrices.
    The word must be specified by its `word number', {\em No} (see section 
    \ref{GdSection:algo.wgen} for details).

    The first form of the ZMW command assumes that there are two generators,
    and the generators must be given as two files names. The second form uses
    the `-g' option to specify the number of generators. In this case, a base
    name is geven on the command line, and the generators are read from
    `{\it Gen}.1', `{\it Gen}.2', \ldots. The word is written to {\em Word},
    if present. If {\it Nsp} is present, the kernel of the word is calculated
    and written to this file.

    {\it No} may be a single number, or a range of numbers in the form
    $a-b$. In the latter case the nullities of all words are calculated and 
    printed, but no output file is written, even if {\it Word} and 
    {\it Nsp} are present.

    ZMW can insert the words into a polynomial before calculating the
    null-space. To use this feature, the polynomial must be appended to 
    the word number, separated by a slash (`/'). After the `/', all
    coefficients, including zeroes, of the polynomial must be specified
    as a comma-separated list. For example,
    \msg{zmw 103/1,1,0,-1 gen1 gen2}
    calculates word number 103, and inserts the result into the 
    polynomial $x^3+x^2-1$.

    Word number 0 is always the first generator. This can be used to insert
    a given matrix into a polynomial. For example
    \msg{zmw 0/1,0,0,1 x y out}
    calculates $x^3+1$ an writes the result to `out' (y is ignored).

    *Mapping single vectors
    Sometimes one only wants to know the image of a given vector under a word.
    This can be done using the `-m' option. For example, to find the
    image of `vec' under word 42, use
    ||    zmw -m 42 gen vec img
    This has the same effect as
    ||    zmw 42 gen w42
    ||    zmu vec w42 img
    but is more efficient, sind unnecessary calculations are avoided.
    Note that -m does not (yet) work with polynomials.



!seealso 
-------------------------------------------------------------------------- }}}*/


